#include "sdcard.h"

//Fat Flags
unsigned char diskInitFlag;
unsigned char diskMountFlag;
unsigned char openFlag;			//is the file open?
unsigned int counter;			//Number of times the file has been written to
unsigned int fileCounter;		//Number of files on the SD card
char dataLogName[26];

//init time
int millis = 0;
int secs = 0;
int mins = 0;
int hours = 13;
int day = 0; //dummy day
int month = 0; //dummy month
int year = 1; //dummy year

/* Read back mode ************************/
#define READBACK 1
#define READBACK_RATE 10 //how many milliseconds in a second?
#define DEBUG 0

#define f_size(fp) ((fp)->fsize) // Give size of file in bytes

// FatFS variables
FIL logFile;
FIL* logPtr = &logFile;
FATFS fs;
FATFS* fsPtr;

//**********************************************************          
//Inits filesystem on the SD card
void init_fs() {
	DSTATUS dstatus;

	dstatus = disk_initialize(0);
	
	#if DEBUG
	if(dstatus == STA_NOINIT) fprintf(stdout, "STA_NOINIT ");
	else if(dstatus == STA_NODISK) fprintf(stdout, "STA_NODISK ");
	else if(dstatus == STA_PROTECT) fprintf(stdout, "STA_PROTECT ");
	#endif
	
	if(dstatus == 0) diskInitFlag = 1; //Disk has been Initialized
	
	FRESULT result;
	//init pointer
	fsPtr = &fs;	
	//set up file system
	result = f_mount(0, fsPtr);
	#if DEBUG
	fprintf(stdout, "Initiating filesystem mount: %d\n\r", result);
	#endif
	
	if(result == 0) diskMountFlag = 1; //Disk has been mounted
	
}

/* User Provided Timer Function for FatFs module */
/* DWORD get_fattime() required for writing to SD card */
/*
	From FatFs specification:
	The return value: DWORD
		31:25 - Year from 1980 (0..127)
		24:21 - Month (1..12)
		20:16 - Day in month (1..31)
		15:11 - Hour (0..23)
		10:5 - Minute (0.59)
		4:0 - Second / 2 (0..29)

*/
DWORD get_fattime()
{
	
	DWORD dword;
	//return a made up date, but a sort of right time
	dword = (((DWORD)(year)) << 25)
	| (((DWORD)(month)) << 21)
	| (((DWORD)(day)) << 16)
	| (((DWORD)(hours)) << 11)
	| (((DWORD)(mins)) << 5)
	| (((DWORD)(secs)) << 1);

	//fprintf(stdout, "dword %d", dword);
	return dword;
}

//Print out file result code
void printFRcode(FRESULT fResult) {
	switch(fResult) {
	case FR_OK:
		fprintf(stdout, "FR OK\n\r");
		break;
	case FR_DISK_ERR:
		fprintf(stdout, "FR Disk Error\n\r");
		break;
	case FR_INT_ERR:
		fprintf(stdout, "FR_INT_ERR\n\r");
		break;
	case FR_NOT_READY:
		fprintf(stdout, "FR_NOT_READY\n\r");
		break;
	case FR_NO_FILE:
		fprintf(stdout, "FR_NO_FILE\n\r");
		break;			
	case FR_NO_PATH:
		fprintf(stdout, "FR_NO_PATH\n\r");
		break;			
	case FR_INVALID_NAME:
		fprintf(stdout, "FR_INVALID_NAME\n\r");
		break;	
	case FR_DENIED:
		fprintf(stdout, "FR_DENIED\n\r");
		break;			
	case FR_EXIST:
		fprintf(stdout, "FR_EXIST\n\r");
		break;			
	case FR_INVALID_OBJECT:
		fprintf(stdout, "FR_INVALID_OBJECT\n\r");
		break;	
	case FR_WRITE_PROTECTED:
		fprintf(stdout, "FR_WRITE_PROTECTED\n\r");
		break;	
	case FR_INVALID_DRIVE:
		fprintf(stdout, "FR_INVALID_DRIVE\n\r");
		break;	
	case FR_NOT_ENABLED:
		fprintf(stdout, "FR_NOT_ENABLED\n\r");
		break;		
	case FR_NO_FILESYSTEM:	
		fprintf(stdout, "FR_NO_FILESYSTEM\n\r");
		break;
	case FR_MKFS_ABORTED:	
		fprintf(stdout, "FR_MKFS_ABORTED\n\r");
		break;
	case FR_TIMEOUT:
		fprintf(stdout, "FR_TIMEOUT\n\r");
		break;			
	default:
		fprintf(stdout, "Unknown FR code\n\r");
		break;
	}
}

//Opens the file to write to
void openLog(unsigned int dayArg,unsigned int hoursArg) {
	FRESULT fileResult;
	unsigned int counter = 0;
	
	//only try to open if the disk has been initialized and mounted
	if(diskInitFlag && diskMountFlag) {
			
	//Try to create a new log file. It will fail if the log exists, keep incrementing log file counter until we succeed
		
		do {
			snprintf(dataLogName, 16, "data%03d.txt", counter);
			fileResult = f_open(logPtr, dataLogName, FA_WRITE | FA_CREATE_NEW);
		} while (fileResult == FR_EXIST && ++counter < 1000);

		#if DEBUG
			fprintf(stdout, "Open Data Log: %s\n\r", dataLogName);
		#endif	
		
		
		/*
		snprintf(dataLogName, 26, "data%02d%02d.txt", dayArg,hoursArg);
		#if DEBUG
			fprintf(stdout, "Open Data Log: %s\n\r", dataLogName);
		#endif			
		fileResult = f_open(logPtr, dataLogName, FA_WRITE | FA_OPEN_ALWAYS);	// Open file with write access
		fileResult = f_lseek(logPtr, f_size(logPtr));	// Move to end of file to append data
		*/
		
		
		if(fileResult) {
			#if DEBUG
			fprintf(stdout, "File Opening Failed: ");
			printFRcode(fileResult);
			#endif
		}
		else {
			#if DEBUG
			fprintf(stdout, "File Opened.\n\r");
			#endif
			openFlag = 1;
		}
	}
	else {
		#if DEBUG
		fprintf(stdout, "Disk not mounted/initialized can't open file\n\r");
		#endif
	}
}

//Closes the file
void closeLog() {
	//only try to close if the file has been opened
	if(openFlag) {
		FRESULT fileResult;
		fileResult = f_close(&logFile);
		if(fileResult) {
			#if DEBUG
			fprintf(stdout, "File Closing Failed: ");
			printFRcode(fileResult);
			#endif
		}
		else {
			#if DEBUG
			fprintf(stdout, "File Close.\n\r");
			#endif
			openFlag = 0;
		}
	}	
}

//Write ADC reading to SD card for testing purposes
void writeSDcardTest(void) {
	FRESULT fileResult;
	char str[20];
	unsigned int length;
	unsigned int bytesWrote;
	
	//Write string to file
	snprintf(str, 20, "%04d:%04d:%04d %04d\n\r", hours,
	mins,
	secs,
	5);
	
	//fprintf(stdout,"Writing file: %s", str); //for debugging
	
	/* check to see if file is open
	should only write to file if it is open */
	if(openFlag && READBACK) {
		length = strlen(str);
		fileResult = f_write(logPtr, str, length, &bytesWrote);
		//sync filesystem
		f_sync(logPtr);
		
		#if DEBUG
		//fprintf(stdout, "Trying to write to file: %d\n\r", fileResult);
		#endif		
		
		if(!fileResult && (length == bytesWrote)) {
			#if DEBUG
			fprintf(stdout, "Successfully written\n\r");
			#endif
			counter++;
		}
	}
	else if(!openFlag && !READBACK) {
		#if DEBUG
		fprintf(stdout, "Unable to write to file since file is not open\n\r");
		#endif
	}
}

//Write ADC reading to SD card
void writeSDcard() {	
	FRESULT fileResult;
	char str[80];
	unsigned int length;
	unsigned int bytesWrote;
	
	#if DEBUG
	fprintf(stdout, "Writing to SD card\n\r");
	#endif
	for(int i = 0;i<dataWriteInterval;i++){
		// Write string to file
		snprintf(str, 80, "Accel1:%d Accel2:%d Force1:%d Force2:%d Force3:%d Gyro1:%d Gyro2:%d \n\r",
		accel1Array[i],accel2Array[i],force1Array[i],force2Array[i],force3Array[i],gyro1Array[i],gyro2Array[i]);
		
		#if DEBUG
		//fprintf(stdout,"str: %s",str);
		#endif
		
		// check to see if file is open should only write to file if it is open 
		if(openFlag && READBACK) {
			length = strlen(str);
			fileResult = f_write(logPtr, str, length, &bytesWrote);
			//sync filesystem
			f_sync(logPtr);	
			
			if(!fileResult && (length == bytesWrote)) {
				#if DEBUG
				//fprintf(stdout, "Successfully written\n\r");
				#endif
				counter++;
			}
		}
		
		else if(!openFlag && !READBACK) {
			#if DEBUG
			fprintf(stdout, "Unable to write to file since file is not open\n\r");
			#endif
		}
	}
}


void readSDcard(void){
	FRESULT fileResult;
	char lineBuf[32];
	
	// Open file bounded by dataLogName
	// IMPORTANT: writeSDcard() must have been called first
	
	fprintf(stdout, "Reading file: %s \n\r", dataLogName);
	fileResult = f_open(logPtr, dataLogName, FA_READ);	
	if(fileResult) {
		#if DEBUG
		fprintf(stdout, "File Opening Failed: ");
		printFRcode(fileResult);
		#endif
	}
	else {
		#if DEBUG
		fprintf(stdout, "File Opened.\n\r");
		#endif
		openFlag = 1;
	}
	
	
	/* check to see if file is open
	should only read to file if it is open */
	
	if(openFlag && READBACK) {
		while(f_gets(lineBuf, 32, logPtr)) {
			fprintf(stdout,"Output from read: %s \n\r", lineBuf);
		}
	}
}

void testSDCard(void){
	openLog(0,0);
	writeSDcardTest();
	closeLog();
	readSDcard();
	closeLog();
}
